<?php

declare(strict_types=1);

namespace app\models\connection;

use yii;
use yii\base\Model;

use app\models\City;

use GuzzleHttp\Client as Guzzle;

use Exception;

class Dellin extends Model
{
    /**
     * Инстанция браузера
     */
    public Guzzle $browser;

    /**
     * Сессия аккаунта
     */
    public string $session;

    public function __construct($config = [])
    {
        parent::__construct($config);

        $this->browser = new Guzzle([
            'base_uri'  => 'https://api.dellin.ru/'
        ]);

        $this->authorization();
    }

    /**
     * Поиск городов
     *
     * @return array|null Найденные города
     */
    public function searchCities(): ?array
    {
        // $this->onReady(function () {
        //     // Запрос городов
        //     $request = $this->browser->post('/v2/public/kladr.json', [
        //         'json' => [
        //             'appkey' => yii::$app->params['dellin']['key'],

        //         ]
        //     ])
        // });
    }

    /**
     * Синхронизация городов
     *
     * @return array|null Найденные города
     *
     * @todo Написать проверку по хешу
     */
    public function syncCities(): ?array
    {
        return $this->ifReady(function () {
            // Запрос ссылки на файл с городам
            // Получаем hash и url ()
            $request = $this->browser->post('/v1/public/cities.json', [
                'json' => [
                    'appkey' => yii::$app->params['dellin']['key'],
                ]
            ]);

            if ($request->getStatusCode() === 200) {
                // Запрос прошел успешно

                // Инициализация
                $response = json_decode((string) $request->getBody(), true);


                if ($response['metadata']['status'] === 200) {
                    // Аутентификация и авторизация пройдены успешно

                    // Запись сессии

                    return $this;
                }

                throw new Exception('Не удалось синхронизировать данные городов с ДеловыеЛинии', $response['metadata']['status']);
            }


            throw new Exception('Не удалось синхронизировать данные городов с ДеловыеЛинии', $request->getStatusCode);

        });
    }

    /**
     * Аутентификация и авторизация
     */
    protected function authorization(): ?self
    {
        return $this->ifReady(function () {
            // Аутентификация и авторизация
            $request = $this->browser->post('/v3/auth/login.json', [
                'json' => [
                    'appkey' => yii::$app->params['dellin']['key'],
                    'login' => yii::$app->params['dellin']['nickname'],
                    'password' => yii::$app->params['dellin']['password']
                ]
            ]);

            if ($request->getStatusCode() === 200) {
                // Запрос прошел успешно

                // Инициализация
                $response = json_decode((string) $request->getBody(), true);

                if ($response['metadata']['status'] === 200) {
                    // Аутентификация и авторизация пройдены успешно

                    // Запись сессии
                    $this->session = $response['data']['sessionID'];

                    return $this;
                }

                throw new Exception('Не удалось авторизироваться в ДеловыеЛинии', $response['metadata']['status']);
            }


            throw new Exception('Не удалось авторизироваться в ДеловыеЛинии', $request->getStatusCode);
        });
    }

    /**
     * Проверка на то, что браузер готов к работе
     *
     * @param callable $function Код к выполнению
     * @param [type] ...$vars Параметры к нему
     *
     * @return mixed Возврат из функции
     */
    protected function ifReady(callable $function, ...$vars): mixed
    {
        if (isset($this->browser)) {
            // Браузер инициализирован

            return $function(...$vars);
        }

        throw new Exception('Браузер не инициализирован', 500);
    }
}
